@@ -129,7 +129,7 @@ class OpenStackSwiftConnection(OpenStackBaseConnection):
             self._service_region = None
 
     def get_endpoint(self, *args, **kwargs):
-        if '2.0' in self._auth_version:
+        if ('2.0' in self._auth_version) or ('3.x' in self._auth_version):
             endpoint = self.service_catalog.get_endpoint(
                 service_type=self._service_type,
                 name=self._service_name,
@@ -137,6 +137,8 @@ class OpenStackSwiftConnection(OpenStackBaseConnection):
         elif ('1.1' in self._auth_version) or ('1.0' in self._auth_version):
             endpoint = self.service_catalog.get_endpoint(
                 name=self._service_name, region=self._service_region)
+        else:
+            endpoint = None
 
         if endpoint:
             return endpoint.url
@@ -310,7 +312,7 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
 
         raise LibcloudError('Unexpected status code: %s' % (response.status))
 
-    def get_container_cdn_url(self, container):
+    def get_container_cdn_url(self, container, ex_ssl_uri=False):
         # pylint: disable=unexpected-keyword-arg
         container_name_encoded = self._encode_container_name(container.name)
         response = self.connection.request('/%s' % (container_name_encoded),
@@ -318,7 +320,10 @@ class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
                                            cdn_request=True)
 
         if response.status == httplib.NO_CONTENT:
-            cdn_url = response.headers['x-cdn-uri']
+            if ex_ssl_uri:
+                cdn_url = response.headers['x-cdn-ssl-uri']
+            else:
+                cdn_url = response.headers['x-cdn-uri']
             return cdn_url
         elif response.status == httplib.NOT_FOUND:
             raise ContainerDoesNotExistError(value='',
